#   Copyright 2015 Rick Harris
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

"""Pyhole Scorekeeper Plugin

Scorekeeper is a fun way to playfully high-five or boo someone.

Each user gets a score which can be incremented like:
    <nick>++

The score can also be decremented like:
    <nick>--

The running highscore can displayed with:
    .score

An individual's score can be displayed with:
    .score <nick>
"""

from pyhole.core import plugin
from pyhole.core import utils


class Scorekeeper(plugin.Plugin):
    """Each user gets a score. Do with it what you want."""
    @plugin.hook_add_msg_regex("^(.+)\+\+")
    @utils.spawn
    def increment_score(self, message, match, **kwargs):
        nick = match.group(1)
        self._adjust_score(message, nick, 1)

    @plugin.hook_add_msg_regex("^(.+)\-\-")
    @utils.spawn
    def decrement_score(self, message, match, **kwargs):
        nick = match.group(1)
        self._adjust_score(message, nick, -1)

    def _adjust_score(self, message, nick, delta):
        score = self._get_score(nick)
        if score is None:
            score = 0
        score += delta
        utils.write_file(self.name, nick, str(score))
        message.dispatch(score)

    def _get_score(self, nick):
        score = utils.read_file(self.name, nick)
        if score is None:
            return None
        try:
            return int(score)
        except ValueError:
            return None

    def _display_highscores(self, message):
        scores = []
        for nick in utils.list_files(self.name):
            score = self._get_score(nick)
            scores.append((score, nick))

        if not scores:
            message.dispatch("No scores yet.")
            return

        message.dispatch("Highscores (Top 5)")
        message.dispatch("==================")
        scores.sort(reverse=True)
        for score, nick in scores[:5]:
            message.dispatch("%s %s" % (str(score).rjust(4),
                                        nick.rjust(13)))

    @plugin.hook_add_command("score")
    @utils.spawn
    def score(self, message, params=None, **kwargs):
        """Display highscores"""
        if params:
            nick = params.strip()
            score = self._get_score(nick)
            if score is None:
                return message.dispatch("No score found for '%s'." % nick)
            else:
                message.dispatch(score)
                return
        else:
            return self._display_highscores(message)



#   Copyright 2015 Jason Meridth
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

"""Pyhole Jira Plugin"""

from jira import JIRA

from pyhole.core import plugin
from pyhole.core import utils


class Jira(plugin.Plugin):
    """Provide access to the Jira API"""

    def __init__(self, irc):
        self.irc = irc
        self.name = self.__class__.__name__

        self.jira = utils.get_config("Jira")
        self.jira_domain = self.jira.get("domain")
        self.jira_username = self.jira.get("username")
        self.jira_password = self.jira.get("password")

        self.jira = JIRA(self.jira_url,
                         basic_auth=(self.jira_username,
                                     self.jira_password))

    @plugin.hook_add_keyword("jira")
    @utils.require_params
    @utils.spawn
    def keyword_jira(self, message, params=None, **kwargs):
        """Retrieve Jira ticket information (ex: jira NCP-1444)"""
        params = utils.ensure_int(params)
        if not params:
            return

        self._find_issue(message, params)

    def _find_issue(self, message, issue_id):
        """Find and display a Jira issue"""

        try:
            issue = self.jira.issue(issue_id)
        except Exception:
            return

        msg = "JIRA %s #%s: %s [S: %s, P: %s, L: %s, A: %s]" % (
            issue.fields.tracker.name, issue.id, issue.fields.summary,
            issue.fields.status.name, issue.fields.priority.name,
            ",".join(issue.fields.labels),
            issue.get("fields", {}).get("assignee", "N/A"))
        url = "https://%s/browse/%s" % (self.jira_url, issue.key)

        message.dispatch("%s %s" % (msg, url))



#   Copyright 2011-2015 Josh Kearney
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

"""Pyhole Kernel.org Plugin"""

import re

from BeautifulSoup import BeautifulSoup

from pyhole.core import plugin
from pyhole.core import utils


class Kernel(plugin.Plugin):
    """Provide access to kernel.org data."""

    @plugin.hook_add_command("kernel")
    @utils.spawn
    def kernel(self, message, params=None, **kwargs):
        """Retrieve the current kernel version (ex: .kernel)"""
        url = "https://www.kernel.org/kdist/finger_banner"
        response = utils.fetch_url(url)
        if response.status_code != 200:
                return

        r = re.compile("(.* mainline .*)")
        m = r.search(response.content)
        kernel = m.group(1).replace("  ", "")
        message.dispatch(kernel)

    @plugin.hook_add_keyword("k")
    @utils.require_params
    @utils.spawn
    def keyword_k(self, message, params=None, **kwargs):
        """Retrieve kernel.org Bugzilla bug information (ex: K12345)"""
        params = utils.ensure_int(params)
        if not params:
            return

        params = {"id": params}
        url = "https://bugzilla.kernel.org/show_bug.cgi"
        response = utils.fetch_url(url, params=params)
        if response.status_code != 200:
            return

        soup = BeautifulSoup(response.content)
        desc = utils.decode_entities(soup.head.title.string)

        try:
            status = soup.find("span", {"id": "static_bug_status"}).string
            status = status.capitalize().split("\n")[0]

            assignee = utils.decode_entities(
                soup.findAll("span", {
                    "class": "vcard"
                })[0].contents[0].string)

            msg = "%s [Status: %s, Assignee: %s] %s"
            message.dispatch(msg % (desc, status, assignee, url))
        except TypeError:
            return

    @plugin.hook_add_msg_regex(
        "https?:\/\/bugzilla\.kernel\.org\/show\_bug\.cgi\?id\=")
    def _watch_for_k_bug_url(self, message, params=None, **kwargs):
        """Watch for kernel.org Bugzilla bug URLs"""
        try:
            bug_id = message.message.split("id=", 1)[1].split(" ", 1)[0]
            self.keyword_k(message, bug_id)
        except TypeError:
            return
